/* Funky3D 1.1
 *
 * Copyright (C) 1991, 1992, 2002, 2003 Heinz van Saanen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Under Windows-Platforms, Funky3D must be compiled as console-application       */
/* ============================================================================== */

#define _BENCHMARK					// BENCHMARK for Benchmarking instead of Demo
#define LCC							// For LCC-Win32 Linker-Libraries; Standard
#define _WATCOM						// For OpenWatcom Linker-Libraries
#ifdef LCC							// Include LCC-Win32 Libraries
	#pragma lib <glu32.lib>
	#pragma lib <glut32.lib>
	#pragma lib <opengl32.lib>
#endif
#ifdef WATCOM						// OpenWatcom Libraries + some "Healings"
	#include <windows.h>
	#include <stdlib.h>
	#pragma library ("\watcom\lib386\nt\glu32.lib");
	#pragma library ("\watcom\lib386\nt\glut32.lib");
	#pragma library ("\watcom\lib386\nt\opengl32.lib");
	#define INTG(d) (x>0.0?floor(d):ceil(d))
	#define FRAC(d) (d-INTG(d))
	#define rint(d) (FRAC(d)>=0.5?INTG(d)+1.0:INTG(d))
	#define M_PI 3.1415969f
	#define M_E 2.7182818f
#endif

// Some #defines with Global-Effects
#define WINSIZE      600			// Size for Main-Window excluding Info-Window
#define DELTA_TRANS  0.05f			// Step-Width for Translations
#define DELTA_ROT    1.0f			// Step-Width for Rotations
#define BOXSIZE      0.2f			// Box-Size for Info-Window
#define SLEN         80				// Lenght of Function-Strings

// Delivers a Random-Float in the Range of -1.0f .. 1.0f
#define RAND() ((rand()/(float)(RAND_MAX)-0.5f)*2.0f)

// The #includes. For _LCC care for #including <windows.h> for
// yourself, when using GLUT-Versions prior to 3.76
#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include "ogl_parser.h"				// Include the Parser-Code

// The Globals for OpenGL-Stuff
GLenum   PolygonMode=GL_LINE;
GLdouble angle = 38.0;
GLfloat  rotX = -12.0f;				// Initial Values for Rotation
GLfloat	rotY = -63.0f;
GLfloat	rotZ =   0.0f;
GLfloat  transX =  0.4f;			// Initial Values For Translation
GLfloat 	transY = -0.1f;
GLfloat	transZ = -5.0f;
GLfloat	_transX =  0.0f;			// Translation Values for Info-Window
GLfloat	_transY =  3.9f;
GLfloat	_transZ =  1.0f;
GLfloat 	crossX = 0.5f;			// Coordinates for Measuring-Cross
GLfloat	crossY = 0.5f;
GLfloat	crossZ = 0.5f;

// Scaling
float sa = 1.0f;					// Scale-Factor over all
float sx = 1.0f;					// Scale-Factors for x,y,z
float sy = 1.0f;
float sz = 1.0f;
float step = 0.05f;					// Step-Widht for the Tesselations

// List-Handling
int update=true;					// Flag if Functions need Update
GLuint TEXT=1;						// OpenGL Display List for Text-Output
GLuint P1=2, P2=3, P3=4;			// Display Lists for drawing Points
GLuint T1=5, T2=6, T3=7;			// Display Lists for drawing Triangles
GLuint Q1=8, Q2=9, Q3=10;			// Display Lists for drawing Quads

// General
int ww, wh;							// Widht/Height for Main-Window
int _ww, _wh;						// Widht/Height for Info-Window
int mode_cross	= 0;				// Show Measuring-Cross
int mode_axes	= 1;				// Show Axes
int mode_draw  = 0;					// 0=Triangles, 1=Lines, 2=Points
int mode_func  = 0;					// 0=fxy1, 1=fxy2, 2=fxy3, 3=fxy1+fxy2+fxy3
int grids      = 25;				// "Gridlines" for Measuring-Cross
GLuint main_window;					// The involved two OpenGL-Windows
GLuint info_window;
GLfloat _bs = BOXSIZE;				// BoxSize in Info-Window
GLfloat _bo = BOXSIZE*1.5f;			// BoxOffset in Info-Window

char f1[SLEN], f2[SLEN];			// Taking the two Functions to draw
char f3[SLEN];						// Maybe for Future-Use

//char title_array[30];
//char *title=title_array;

// Ansi-compliant Timer-Stuff
#define START	t=clock(); while(t==clock()); t++
#define STOP	t=clock()-t; seconds=t/((double)CLOCKS_PER_SEC)
#ifdef BENCHMARK						// If BENCHMARK is #defined ...
#define WAIT							// Skip Waiting
#else
#define WAIT(sec)  Wait((float)sec)		// Else Wait for sec seconds
#endif

clock_t t;
float seconds;


/* ************************************************************************* */
/* Some Auxiliary-Functions                                                  */
/* ************************************************************************* */


// Wait for Seconds
// ---------------------------------------------------------------------------
static void Wait(float sec);
static void Wait(float sec) {
clock_t t,h;
float s;
	h=clock();
	while(1) {
		t=clock();
		s=(t-h)/((float)CLOCKS_PER_SEC);
		if(s>sec) break;
	}
} // End of Wait()


// Reset the View to original Values
// ---------------------------------------------------------------------------
static void Set_Default(void);
static void Set_Default(void) {
	PolygonMode=GL_LINE;
	angle = 38.0f;
	rotX = -12.0f;				// Initial Values for Rotation
	rotY = -63.0f;
	rotZ =   0.0f;
	transX =  0.4f;				// Initial Values For Translation
	transY = -0.1f;
	transZ = -5.0f;
	crossX = 0.5f;				// Coorinates for Measuring-Cross
	crossY = 0.5f;
	crossZ = 0.5f;
	sa = 1.0f;					// Scale-Factor over all
	sx = 1.0f;					// Scale-Factors for x,y,z
	sy = 1.0f;
	sz = 1.0f;
	step = 0.05f;				// Step-Widht for Tesselation
	update = true;				// Drawing Functions needs Update
} // End of Set_Default()


// Save current State to RAM
// ---------------------------------------------------------------------------
static void Save(void);
static void Save(void) {
	PolygonMode=GL_LINE;
	angle = 38.0f;
	rotX = -12.0f;				// Initial Values for Rotation
	rotY = -63.0f;
	rotZ =   0.0f;
	transX =  0.4f;				// Initial Values For Translation
	transY = -0.1f;
	transZ = -5.0f;
	crossX = 0.5f;				// Coorinates for Measuring-Cross
	crossY = 0.5f;
	crossZ = 0.5f;
	sa = 1.0f;					// Scale-Factor over all
	sx = 1.0f;					// Scale-Factors for x,y,z
	sy = 1.0f;
	sz = 1.0f;
	step = 0.05f;				// Step-Widht for Tesselation
	update = true;				// Drawing Functions needs Update
} // End of Save()


// Calculating Parser-Values for Function 1
// ---------------------------------------------------------------------------
static float fxy1(float _x, float _y);
static float fxy1(float _x, float _y) {
	x=_x*sx;
	y=_y*sy;
	return (parse(f1)*sz);
} // End of fxy1()


// Calculating Parser-Values for Function 2
// ---------------------------------------------------------------------------
static float fxy2(float _x, float _y);
static float fxy2(float _x, float _y) {
	x=_x*sx;
	y=_y*sy;
	return (parse(f2)*sz);
} // End of fxy2()


// Calculating Parser-Values for Function 3
// ---------------------------------------------------------------------------
static float fxy3(float _x, float _y);
static float fxy3(float _x, float _y) {
	x=_x*sx;
	y=_y*sy;
	return (parse(f3)*sz);
} // End of fxy3()


// Solves 3 Functions numerically via Random-Search
// ---------------------------------------------------------------------------
static void Solve(void);
static void Solve(void) {
int i=0, steps=50000;
float _x, _y, _z, dx, dy, hz, eps=1e-6f;
int ready=false;
	parse(f1); if(elevel) { printf("Cannot solve because of Failure in F1\n"); return; }
	parse(f2); if(elevel) { printf("Cannot solve because of Failure in F2\n"); return; }
	parse(f3); if(elevel) { printf("Cannot solve because of Failure in F3\n"); return; }
	_x=crossX;
	_y=crossY;
	printf("Start solving Equation-System at x=%f y=%f\n", _x, _y);
	// Calculate the Difference of the Z-Values from X,Y-Cross-Value
	_z =fabs(fxy2(_x, _y)-fxy1(_x, _y));
	_z+=fabs(fxy3(_x, _y)-fxy1(_x, _y));
	_z+=fabs(fxy3(_x, _y)-fxy2(_x, _y));
	do {
		dx=RAND()*eps;										// Generate a new Random-Vector
		dy=RAND()*eps;										// With Length eps
		hz =fabs(fxy2(_x+dx, _y+dy)-fxy1(_x+dx, _y+dy));	// Calculate Z-Differences with it
		hz+=fabs(fxy3(_x+dx, _y+dy)-fxy1(_x+dx, _y+dy));
		hz+=fabs(fxy3(_x+dx, _y+dy)-fxy2(_x+dx, _y+dy));
		while(hz<_z) {
			i++;
		   if(i>steps) break;
			if(hz<eps)  { ready=true; break; }
			_z=hz;
			dx*=2.0f;	// Resize Random-Vector dynamically
			dy*=2.0f;
			_x+=dx;
			_y+=dy;
			hz =fabs(fxy2(_x+dx, _y+dy)-fxy1(_x+dx, _y+dy));
			hz+=fabs(fxy3(_x+dx, _y+dy)-fxy1(_x+dx, _y+dy));
			hz+=fabs(fxy3(_x+dx, _y+dy)-fxy2(_x+dx, _y+dy));
		}
		if(ready==true) break;
		_x-=dx;			// Last Random-Search wasn't successfull; Take back one Step
		_y-=dy;
		_z =fabs(fxy2(_x, _y)-fxy1(_x, _y));
		_z+=fabs(fxy3(_x, _y)-fxy1(_x, _y));
		_z+=fabs(fxy3(_x, _y)-fxy2(_x, _y));
		i++;
	} while (i<steps);
	crossX=_x;
	crossY=_y;
	glutSetWindow(info_window);
	glutPostRedisplay();
	glutSetWindow(main_window);
	glutPostRedisplay();
	printf("Result%s realiable. %d Iterations done\n", (ready==true)?"":" maybe NOT", i);
	printf("x=%f y=%f z=%f\n\n", _x*sx, _y*sy, fxy1(_x,_y)/sz);
} // End of Solve()


// Show Help-Text
// ---------------------------------------------------------------------------
static void Show_Help(void);
static void Show_Help(void) {
	printf("FUNKY3D USAGE OF MOUSE/INFO-SCREEN\n");
	printf("===============================================================================\n");
	printf("Scale X -/+   : Scales X-Axis with Factor 2; same for Y and Z\n");
	printf("  X+Y+Z -/+   : Scales all Axes with same Factor\n");
	printf("   Grid -/+   : Change Size of Tesselation-Grid\n");
	printf("   Zoom -/+   : Zoom Out/In\n");
	printf("       x1 =   : Actual Size of biggest positive X-Coordinate; same for Y and Z\n");
	printf("       cx =   : X-Coordinate of Cross-Position for Function 1; same for Y and Z\n");
	printf("Function 1    : Actual 1st Function shown. Analogue for 2 and 3\n");
	printf("Moving Cross  : Press left Mouse-Button and move Mouse\n");
	printf("\nFUNKY3D USAGE OF KEYBOARD\n");
	printf("===============================================================================\n");
	printf("1..3          : Input Functions 1-3 in Console Window\n");
	printf("Cursors       : Rotate Functions arround X- or Y-Axis\n");
	printf("A/a           : Toggle Axes On/Off\n");
	printf("C/c           : Toggle Cross On/Off\n");
	printf("D/d           : Run Demo\n");
	printf("G/g           : Increase/Decrease Tesselation-Grids\n");
	printf("M/m           : Toggle Function Drawing-Mode Triangles/Quads/Points\n");
	printf("R/r           : Reset to initial View\n");
	printf("S/s           : Solve Equation-System numerically\n");
	printf("\nPress <Return> to continue\n"); getchar();
	printf("X/x           : Scale X-Axis bigger/smaller; same for Y and Z\n");
	printf("Ins-Del-Block : Ins/Del/Home/End/PgUp/PgDwn = more Moving&Rotating\n");
	printf("Space         : Toggle Line/Filled-Mode\n");
	printf("+/-           : Zoom In/Out\n");
	printf("\n(c) Heinz van Saanen 2003\n");
} // End of Show_Help()


/* ************************************************************************* */
/* Starting OpenGL-Stuff                                                     */
/* ************************************************************************* */


// Drawing Strings
// ---------------------------------------------------------------------------
static void Draw_String(GLfloat x, GLfloat y, GLfloat z, char* format, ...);
static void Draw_String(GLfloat x, GLfloat y, GLfloat z, char* format, ...) {
va_list args;
char buffer[255], *s;

	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);
	glRasterPos3f(x,y,z);
	for(s=buffer; *s; s++)
		glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *s);
} // End of Draw_String()


// Drawing Check-Boxes in Info-Window
// ---------------------------------------------------------------------------
static void _Draw_2Boxes(GLfloat x, GLfloat y);
static void _Draw_2Boxes(GLfloat x, GLfloat y) {
	// _bs = BoxSize
	glColor3f(0.85f, 0.85f, 1.0f);
	glBegin(GL_QUADS);
		glVertex2f(x    , y    ); glVertex2f(x    , y+_bs);
		glVertex2f(x+_bs, y+_bs); glVertex2f(x+_bs, y    );
	glEnd();
	glColor3f(0.0f, 0.0f, 0.0f);			// Set Color Black
	Draw_String(x+_bs*0.4f, y+_bs*0.4f, 0.0f,"-");

	x=x+_bs;
	glColor3f(0.6f, 0.97f, 0.97f);
	glBegin(GL_QUADS);
		glVertex2f(x    , y    ); glVertex2f(x    , y+_bs);
		glVertex2f(x+_bs, y+_bs); glVertex2f(x+_bs, y    );
	glEnd();

	glColor3f(0.0f, 0.0f, 0.0f);			// Set Color Black
	Draw_String(x+_bs*0.4f, y+_bs*0.4, 0.0f, "+");
} // End of _Draw_2Boxes()


// Show Values in Info-Window
// ---------------------------------------------------------------------------
static void _Show_Values(GLfloat x, GLfloat y);
static void _Show_Values(GLfloat x, GLfloat y) {
char s[20];

	// Shows current X-Scale-Factor
	if((sx>100000.0)||(sx<0.00001)) {
	sprintf(s,"x1 = %.4e", sx); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }
	else {
	sprintf(s,"x1 = %.5f", sx); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }

	// Shows current Y-Scale-Factor
	if((sy>100000.0)||(sy<0.00001)) {
	sprintf(s,"y1 = %.4e", sy); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }
	else {
	sprintf(s,"y1 = %.5f", sy); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }

	// Shows current Z-Scale-Factor
	if((sz>100000.0)||(sz<0.00001)) {
	sprintf(s,"z1 = %.4e", sz); Draw_String(x, y, 0.0f, s); y-=_bo; }
	else {
	sprintf(s,"z1 = %.5f", sz); Draw_String(x, y, 0.0f, s); y-=_bo; }

	// Shows current X-Value for Cross
	if(abs((crossX*sx))>100000.0) {
	sprintf(s,"cx = %.4e", crossX*sx); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }
	else {
	sprintf(s,"cx = %.5f", crossX*sx); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }

	// Shows current Y-Value for Cross
	if(abs((crossY*sy))>100000.0) {
	sprintf(s,"cy = %.4e", crossY*sy); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }
	else {
	sprintf(s,"cy = %.5f", crossY*sy); Draw_String(x, y, 0.0f, s); y-=_bo/2.0f; }

	// Calculate/Shows current Z-Value for Cross
	crossZ=fxy1(crossX, crossY)/sz;
	if(abs((crossZ))>100000.0) {
	sprintf(s,"cz = %.4e", crossZ); Draw_String(x, y, 0.0f, s); y-=_bo; }
	else {
	sprintf(s,"cz = %.5f", crossZ); Draw_String(x, y, 0.0f, s); y-=_bo; }

	// Shows current Functions
	Draw_String(x,y,0.0f,"Function 1:"); y-=_bo/2.0f; Draw_String(x, y, 0.0f, f1); y-=_bo;
	Draw_String(x,y,0.0f,"Function 2:"); y-=_bo/2.0f; Draw_String(x, y, 0.0f, f2); y-=_bo;
	Draw_String(x,y,0.0f,"Function 3:"); y-=_bo/2.0f; Draw_String(x, y, 0.0f, f3); y-=_bo;

	// Shows current Function(s) choosen
	Draw_String(x,y,0.0f,"(T)oggle Drawing:");  y-=_bo/2.0f;
	if(mode_func==0) Draw_String(x, y, 0.0f, "F1()");
	if(mode_func==1) Draw_String(x, y, 0.0f, "F2()");
	if(mode_func==2) Draw_String(x, y, 0.0f, "F3()");
	if(mode_func==3) Draw_String(x, y, 0.0f, "F1()+F2()+F3()");
	y-=_bo; Draw_String(x, y, 0.0f, "F1=Help D=Demo");
} // End of _Show_Values()


// Draws Points
// ---------------------------------------------------------------------------
static void Draw_Points(void);
static void Draw_Points(void) {
float x, y, z;
	if(*f1!='\0' && (mode_func==0 || mode_func==3)) {
		if(update==true) {
			glNewList(P1, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_POINTS);
			for(x=-1.0f; x<1.0f; x+=step)
		  		for(y=-1.0; y<1.0; y+=step) {
					z=fxy1(x, y);
					if(!elevel) {
						glColor3f(0.7f, y, z);
						glVertex3f(x, y, z);
					}
				}
			glEnd();
			glEndList();
		}
		else
			glCallList(P1);
	}
	if(*f2!='\0' && (mode_func==1 || mode_func==3)) {
		if(update==true) {
			glNewList(P2, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_POINTS);
			for(x=-1.0f; x<1.0f; x+=step)
			  	for(y=-1.0; y<1.0; y+=step) {
					z=fxy2(x, y);
					if(!elevel) {
						glColor3f(z, -y, 0.7f);
						glVertex3f(x, y, z);
					}
				}
			glEnd();
			glEndList();
		}
		else
			glCallList(P2);
	}
	if(*f3!='\0' && (mode_func==2 || mode_func==3)) {
		if(update==true) {
			glNewList(P3, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_POINTS);
			for(x=-1.0f; x<1.0f; x+=step)
			  	for(y=-1.0; y<1.0; y+=step) {
					z=fxy3(x, y);
					if(!elevel) {
						glColor3f(y, 0.6f, z);
						glVertex3f(x, y, z);
					}
				}
			glEnd();
			glEndList();
		}
		else
			glCallList(P3);
	}
	update=false;
} // End of Draw_Points()


// Draws Quads
// ---------------------------------------------------------------------------
static void Draw_Quads(void);
static void Draw_Quads(void) {
float x, y, z1, z2, z3, z4;
int err;
	if(*f1!='\0' && (mode_func==0 || mode_func==3)) {
		if(update==true) {
			glNewList(Q1, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_QUADS);
			for(x=-1.0; x<1.0; x+=step) {
				for(y=-1.0; y<1.0; y+=step) {
					err=0;
					z1=fxy1(x, y); 				err=err||elevel;
					z2=fxy1(x,y+step);			err=err||elevel;
					z3=fxy1(x+step, y+step);	err=err||elevel;
					z4=fxy1(x+step, y); 			err=err||elevel;
					if(!err) {
						glColor3f(0.7f, y, z1);
						glVertex3f(x, y, z1);
						glVertex3f(x, y+step, z2);
						glVertex3f(x+step, y+step, z3);
						glVertex3f(x+step, y, z4);
					}
				}
			}
			glEnd();
			glEndList();
		}
		else
			glCallList(Q1);
	}
	if(*f2!='\0' && (mode_func==1 || mode_func==3)) {
		if(update==true) {
			glNewList(Q2, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_QUADS);
			for(x=-1.0; x<1.0; x+=step) {
				for(y=-1.0; y<1.0; y+=step) {
					err=0;
					z1=fxy2(x, y); 				err=err||elevel;
					z2=fxy2(x,y+step);			err=err||elevel;
					z3=fxy2(x+step, y+step);	err=err||elevel;
					z4=fxy2(x+step, y); 			err=err||elevel;
					if(!err) {
						glColor3f(z1, -y, 0.7f);
						glVertex3f(x, y, z1);
						glVertex3f(x, y+step, z2);
						glVertex3f(x+step, y+step, z3);
						glVertex3f(x+step, y, z4);
					}
				}
			}
			glEnd();
			glEndList();
		}
		else
			glCallList(Q2);
	}
	if(*f3!='\0' && (mode_func==2 || mode_func==3)) {
		if(update==true) {
			glNewList(Q3, GL_COMPILE_AND_EXECUTE);
			glBegin(GL_QUADS);
			for(x=-1.0; x<1.0; x+=step) {
				for(y=-1.0; y<1.0; y+=step) {
					err=0;
					z1=fxy3(x, y); 				err=err||elevel;
					z2=fxy3(x,y+step);			err=err||elevel;
					z3=fxy3(x+step, y+step);	err=err||elevel;
					z4=fxy3(x+step, y); 			err=err||elevel;
					if(!err) {
						glColor3f(y, 0.6f, z1);
						glVertex3f(x, y, z1);
						glVertex3f(x, y+step, z2);
						glVertex3f(x+step, y+step, z3);
						glVertex3f(x+step, y, z4);
					}
				}
			}
			glEnd();
			glEndList();
		}
		else
			glCallList(Q3);
	}
	update=false;
} // End Draw_Quads()


//Draws Triangles
// ---------------------------------------------------------------------------
static void Draw_Triangles(void);
static void Draw_Triangles(void) {
float x, y, z;
	if(*f1!='\0' && (mode_func==0 || mode_func==3)) {
		if(update==true) {
			glNewList(T1, GL_COMPILE_AND_EXECUTE);
			for(x=-1.0; x<1.0; x+=step) {
				glBegin(GL_TRIANGLE_STRIP);
				for(y=-1.0; y<1.0; y+=step) {
					z=fxy1(x, y);
					glColor3f(0.7f, y, z);
					if(!elevel) glVertex3f(x, y, z);
					z=fxy1(x+step, y);
					if(!elevel) glVertex3f(x+step, y, z);
				}
				glEnd();
			}
			glEndList();
		}
		else
			glCallList(T1);
	}
	if(*f2!='\0' && (mode_func==1 || mode_func==3)) {
		if(update==true) {
			glNewList(T2, GL_COMPILE_AND_EXECUTE);
			for(x=-1.0; x<1.0; x+=step) {
				glBegin(GL_TRIANGLE_STRIP);
				for(y=-1.0+step; y<1.0; y+=step) {
					z=fxy2(x, y);
					glColor3f(z, -y, 0.7f);
					if(!elevel) glVertex3f(x, y, z);
					z=fxy2(x+step, y);
					if(!elevel) glVertex3f(x+step, y, z);
				}
				glEnd();
			}
			glEndList();
		}
		else
			glCallList(T2);
	}
	if(*f3!='\0' && (mode_func==2 || mode_func==3)) {
		if(update==true) {
			glNewList(T3, GL_COMPILE_AND_EXECUTE);
			for(x=-1.0; x<1.0; x+=step) {
				glBegin(GL_TRIANGLE_STRIP);
				for(y=-1.0+step; y<1.0; y+=step) {
					z=fxy3(x, y);
					glColor3f(y, 0.6f, z);
					if(!elevel) glVertex3f(x, y, z);
					z=fxy3(x+step, y);
					if(!elevel) glVertex3f(x+step, y, z);
				}
				glEnd();
			}
			glEndList();
		}
		else
			glCallList(T3);
	}
	update=false;
} // End of Draw_Triangles()


// Draws Axes-Arrows
// ---------------------------------------------------------------------------
static void Draw_Arrows(void);
static void Draw_Arrows(void) {
GLfloat delta=0.025f;
	glBegin(GL_TRIANGLES);
		glVertex3f(1.5f, -delta, 0.0f);
		glVertex3f(1.5f,  delta, 0.0f);
		glVertex3f(1.5f+2.0f*delta, 0.0f, 0.0f);

		glVertex3f(-delta, 1.5f, 0.0f);
		glVertex3f( delta, 1.5f, 0.0f);
		glVertex3f(  0.0f, 1.5f+2.0f*delta, 0.0f);

		glVertex3f( -delta, 0.0f, 1.5f);
		glVertex3f(  delta, 0.0f, 1.5f);
		glVertex3f(   0.0f, 0.0f, 1.5f+2.0f*delta);
	glEnd();
} // End of Draw_Arrows()


// Draws Axes
// ---------------------------------------------------------------------------
static void Draw_Axes(void);
static void Draw_Axes(void) {
	glColor3f(1.0f,1.0f,1.0f);				// Set Color White
	glBegin(GL_LINES);

		glVertex3f( 0.0f, 0.0f, 0.0f);
		glVertex3f( 1.5f, 0.0f, 0.0f);

		glVertex3f( 0.0f, 0.0f, 0.0f);
		glVertex3f( 0.0f, 1.5f, 0.0f);

		glVertex3f( 0.0f, 0.0f, 0.0f);
		glVertex3f( 0.0f, 0.0f, 1.5f);
	glEnd();

	Draw_String( 1.5f, -0.1f, -0.0f, "x");
	Draw_String( 0.1f,  1.5f, -0.0f, "y");
	Draw_String(-0.1f, -0.1f,  1.5f, "f(xy)");
	Draw_Arrows();
}


// Draws Measuring-Cross
// ---------------------------------------------------------------------------
static void Draw_Cross(void);
static void Draw_Cross(void) {
GLfloat delta=0.1f;							// Cross-Size
	glColor3f(0.0f, 1.0f, 1.0f);			// Set Color White
	glBegin(GL_LINES);
		glVertex2f(crossX-delta,  crossY);
		glVertex2f(crossX+delta,  crossY);
		glVertex2f(crossX,        crossY-delta);
		glVertex2f(crossX,        crossY+delta);
		glVertex3f(crossX,        crossY, -2.0f);
		glVertex3f(crossX,        crossY,  2.0f);
	glEnd();
} // End of Draw_Cross()


// Intitializing States for Main-Window
// ---------------------------------------------------------------------------
static void Init(void);
static void Init(void) {
	glEnable(GL_DEPTH_TEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
} // End of Init()


// Intitializing States for Info-Window
// ---------------------------------------------------------------------------
static void _Init(void);
static void _Init(void) {
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);						// Enable smooth Shading
	glClearColor(0.96f, 0.96f, 0.98f, 0.5f);		// Set Background-Color
} // End of _Init()


// Handles resized Main-Window
// ---------------------------------------------------------------------------
static void Reshape(int width, int height);
static void Reshape(int width, int height) {
	ww=width;
	wh=height;
	// Set Viewport to whole ClientArea of Main-Window
	glViewport(0, 0, (GLint)width, (GLint)height);

	// Initialize Projection-Matrix
	// Ratio Width:Height=1:1, Clipping for z<1 && z>200
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// angle, aspect, near Clip, far Clip
	gluPerspective(angle, (float)width/(float)height, 0.5, 200.0);

	// Set Modelview-Matrix to actual Matrix
	glMatrixMode(GL_MODELVIEW);
} // End of Reshape()


// Handles resized Info-Window
// ---------------------------------------------------------------------------
static void _Reshape(int width, int height);
static void _Reshape(int width, int height) {
	_ww=width;
	_wh=height;

	glViewport(0, 0, (GLint)width, (GLint)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// 50.0 is "magic" value
	gluPerspective(50.0f, (GLfloat)width/(GLfloat)height, 9.0f, -100.0f);
	glMatrixMode(GL_MODELVIEW);
} // End of _Reshape()


// Draws Info-Screen
// ---------------------------------------------------------------------------
static void _Info_Screen(void);
static void _Info_Screen(void) {
GLfloat s  =  0.0f;			// y=0.0 for Box-Startvalue
GLfloat dx = -0.65f;		// delta x; moves OutputString left/right
GLfloat dy =  0.06f;		// delta y; moves OutputString up/down

	// _bo = BoxOffset, 0.1 = magic Offset
	Draw_String( dx, s+dy, 0.0f, "Scale X"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	Draw_String( dx, s+dy, 0.0f, "Scale Y"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	Draw_String( dx, s+dy, 0.0f, "Scale Z"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	Draw_String( dx, s+dy, 0.0f, "  X+Y+Z"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	Draw_String( dx, s+dy, 0.0f, "   Grid"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	Draw_String( dx, s+dy, 0.0f, "   Zoom"); _Draw_2Boxes( 0.0f, s); s-=_bo;
	_Show_Values(dx, s);
} // End of _Info_Screen()


// Central Drawing-Routine for Main-Window
// ---------------------------------------------------------------------------
static void Display(void);
static void Display(void) {
	// Delete FrameBuffer && Z-Buffer
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	// Initialize Modelview-Matrix
	glLoadIdentity ();

	// Do Model-Transformations
	glTranslatef(0.0, 0.0 ,transZ);    		// Translate in Z-Direction
	glRotatef(rotX, 1.0, 0.0, 0.0);  		// Rotate around X-Axis
	glRotatef(rotY, 0.0, 1.0, 0.0);  		// Rotate around Y-Axis
	glRotatef(rotZ, 0.0, 0.0, 1.0);  		// Rotate around Z-Axis
	glTranslatef(transX, transY , -0.5); 	// Translate all with -0.5

	glColor3f(1.0f, 1.0f, 1.0f);

	// ... and draw under current Model-Transformation
	if(mode_axes==1)  Draw_Axes();
	if(mode_draw==0)  Draw_Triangles();
	if(mode_draw==1)  Draw_Quads();
	if(mode_draw==2)  Draw_Points();
	if(mode_cross==1) Draw_Cross();

	// Copy Back-Buffer to Front-Buffer finally
	glutSwapBuffers();
} // End of Display()


// Central Drawing-Routine for Info-Window
// ---------------------------------------------------------------------------
static void _Display(void);
static void _Display(void) {
	// Delete FrameBuffer unf Z-Buffer
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	// Initialize Modelview-Matrix
	glLoadIdentity ();
	// Position Camera
	gluLookAt(0.0f, 0.0f, 10.0f,				// x,y,z Eye
				 0.0f, 0.0f, 0.0f,				// x,y,z LookAt
				 0.0f, 1.0f, 0.0f);				// Up-direction
	// Do Modeltransformation ...
	glTranslatef(_transX, _transY, _transZ);	// Translations
	glColor3f(0.0f, 0.0f, 0.0f);				// Set Color Black
	// Draw now
	_Info_Screen();
	// Copy Back-Buffer to Front-Buffer finally
	glutSwapBuffers();
} // End of _Display()


// Creates a demotext; shows principal Usage of precompiled display lists
// ---------------------------------------------------------------------------
static void Create_List(GLuint list_nr);
static void Create_List(GLuint list_nr) {
float start=0.93f, step=0.08f, _y=0.94f;
	glNewList(list_nr, GL_COMPILE);			// Create a List instead of immediate Executing
		glDisable(GL_DEPTH_TEST);			// Some gl-commands to be stored
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'F'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'u'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'n'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'k'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'y'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'3'); start+=step;
		glRasterPos2f(start, _y); glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'D'); start+=step;
	glEndList();
} // End of Create_List()


// The code for demo || benchmark
// ---------------------------------------------------------------------------
static void Demo(void);
static void Demo(void) {
char h1[SLEN], h2[SLEN], h3[SLEN];					// For Saving Functions
float delay=1.5f;
int i;
	printf("Quick overview over Funky-3D capabilities\n\n");
	strcpy(h1,f1);										// Save current Functions
	strcpy(h2,f2);
	strcpy(h3,f3);
	strcpy(f1,"COS(4X^2+4Y^2)/(1.5+X+3Y^2)-0.5\0");	// Introduce new Ones
	strcpy(f2,"(X+0.3)*Y-0.25\0");
	strcpy(f3,"XXY-0.4\0");
	glutSetWindow(info_window);						// Do some Updating
	_Display();
	glutSetWindow(main_window);
	glutPostRedisplay();
	glPushMatrix();									// Save the current OpenGL-Context
	Set_Default();									// Reset to Default-Values
	START;											// Start Ansi-Timer

	// Some Examples
	printf("Function 1 pure\n");
	PolygonMode=GL_LINE; glPolygonMode(GL_FRONT_AND_BACK, PolygonMode);
	mode_func=0; mode_draw=0; mode_axes=0; mode_cross=0;
	Display(); WAIT(delay);

	printf("Function 1 + Axes\n");
	mode_axes=1;
	Display(); WAIT(delay);

	printf("Function 1 + Axes + Measuring-Cross\n");
	mode_cross=1;
	Display(); WAIT(delay);

	printf("Function 1 with less Grids\n");
	mode_axes=0; mode_cross=0; step=0.150f;
	update=true; Display(); WAIT(delay);

	printf("Function 1 with more Grids\n");
	step=0.050f;
	update=true; Display(); WAIT(delay);

	printf("Function 2 with Quad-Tesselation\n");
	mode_func=1; mode_draw=1; step=0.05f;
	update=true; Display(); WAIT(delay);

	printf("Function 3 with Point-Tesselation\n");
	mode_func=2; mode_draw=2; step=0.02f;
	update=true; Display(); WAIT(delay);

	printf("All Functions in a filled style\n");
	mode_func=3; mode_draw=0; mode_axes=1; mode_cross=1; step=0.05f;
   PolygonMode=GL_FILL; glPolygonMode(GL_FRONT_AND_BACK, PolygonMode);
	update=true; Display(); WAIT(delay);

	printf("Move Cross \"near\" a Solution of the equation-System\n");
	for(i=0; i<10; i++) {
		crossX+=0.01f;
		crossY-=0.07f;
		update=true; Display();
	}
	WAIT(delay);

	printf("Solve Equation-System numerically now\n\n");
	Solve();
	update=true; Display(); WAIT(delay);
	PolygonMode=GL_LINE; glPolygonMode(GL_FRONT_AND_BACK, PolygonMode);

	printf("Some Rotations & Movement\n");
	mode_func=1;
	for(i=0; i<35; i++) {
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();
		glTranslatef(0.0f, 0.0f ,transZ);
		glRotatef(rotY, 0.0f, 1.0f, 0.0f);
		glRotatef(rotX, 1.0f, 0.0f, 0.0f);
		glRotatef(rotZ, 0.0f, 0.0f, 1.0f);
		glTranslatef(transX, transY , -0.5f);
  		update=true; Display();
  		glutSwapBuffers();
  		transX -= DELTA_TRANS;
  		transY += DELTA_TRANS;
  		angle-=2.5f; Reshape(ww,wh);
	}

	printf("Doing some Scaling\n");
	Set_Default(); mode_func=0; mode_cross=0; sz=0.0; Init(); Reshape(ww,wh);
	for(i=0; i<30; i++) {
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();
		glTranslatef(0.0f, 0.0f, transZ);
		glRotatef(rotY, 0.0f, 1.0f, 0.0f);
		glRotatef(rotX, 1.0f, 0.0f, 0.0f);
		glRotatef(rotZ, 0.0f, 0.0f, 1.0f);
		glTranslatef(transX, transY , -0.5f);
		update=true; Display();
		glutSwapBuffers();
		sz+=0.025f;
	}

	printf("Text-Output using OpenGL-display lists\n");
	glPushMatrix();						// Create a new Context
	Create_List(TEXT);					// Create a List
	for(i=0; i<10; i++) {				// Run it for some Times
		glColor3f(0.0,i/10.0,i/10.0);
		glCallList(1);
		glTranslatef(0.0,-0.1,0.0);
		glutSwapBuffers();
	}
	glPopMatrix();						// Load last Context back
	glPopMatrix();						// Restore to the original OpenGL-context
	WAIT(delay);
	Set_Default();						// Set back to Default-Values
	glEnable(GL_DEPTH_TEST);
	strcpy(f1,h1);						// Restore original Functions before calling Demo()
	strcpy(f2,h2);
	strcpy(f3,h3);
	STOP;								// Get the Time needed for Execution

	// Give some final Text-Outputs now
	printf("Demo/Benchmark finished after %.2f seconds\n", seconds);
	printf("Funky3D returned to Default-State\n");
} // End of demo() || Benchmark-Code


// Evaluating Mouse-Clicks in Info-Window
// ---------------------------------------------------------------------------
static void _Check_Mouse_Results(GLdouble x, GLdouble y);
static void _Check_Mouse_Results(GLdouble x, GLdouble y) {

	// Check 1.st Box, Scale X
	if( x> 0.0 && x<    _bs && y>0.0 && y<_bs)  { update=true; sx/=2.0f; }
	if( x>  _bs && x<2.0*_bs && y>0.0 && y<_bs) { update=true; sx*=2.0f; }

   // Check 2.nd Box, Scale Y
	if( x> 0.0 && x<    _bs && y>-1.5*_bs && y<-0.5*_bs)  { update=true; sy/=2.0f; }
	if( x>  _bs && x<2.0*_bs && y>-1.5*_bs && y<-0.5*_bs) { update=true; sy*=2.0f; }

	// Check 3.rd Box, Scale Z
	if( x> 0.0 && x<    _bs && y>-3.0*_bs && y<-2.0*_bs)  { update=true; sz/=2.0f; }
	if( x>  _bs && x<2.0*_bs && y>-3.0*_bs && y<-2.0*_bs) { update=true; sz*=2.0f; }

	// Check 4.th Box, Scale X+Y+Z
	if( x> 0.0 && x<    _bs && y>-4.5*_bs && y<-3.5*_bs)  { update=true; sa/=2.0f; sx=sy=sz=sa; }
	if( x>  _bs && x<2.0*_bs && y>-4.5*_bs && y<-3.5*_bs) { update=true; sa*=2.0f; sx=sy=sz=sa; }

	// Check 5.th Box, Scale Tesselation-Grid
	if( x> 0.0 && x<    _bs && y>-6.0*_bs && y<-5.0*_bs)  { update=true; step*=2.0f; }
	if( x>  _bs && x<2.0*_bs && y>-6.0*_bs && y<-5.0*_bs) { update=true; step/=2.0f; }

	// Check 5.th Box, Zoom
	if( x> 0.0 && x<    _bs && y>-7.5*_bs && y<-6.5*_bs) {
		glutSetWindow(main_window);
		angle+=2.0; Reshape(ww,wh);
		glutSetWindow(info_window); }
	if( x>  _bs && x<2.0*_bs && y>-7.5*_bs && y<-6.5*_bs) {
		glutSetWindow(main_window);
		angle-=2.0; Reshape(ww,wh);
		glutSetWindow(info_window); }
} // End of _Check_Mouse_Results()


// Detects Mouse-Click in Info-Window
// ---------------------------------------------------------------------------
static void _Static_Mouse(int button, int state, int x, int y);
static void _Static_Mouse(int button, int state, int x, int y) {
GLdouble pm[16], mvm[16], ox, oy, oz;
GLint viewport[4];
GLint realy;

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, mvm);
	glGetDoublev(GL_PROJECTION_MATRIX, pm);

	// Real-World-Coordinates available when Z==0 only
	if(button==GLUT_LEFT_BUTTON && state==GLUT_DOWN) {
		realy=viewport[3]-(GLint)y-1;
		gluUnProject(x,realy,0.0,
		             mvm,
						 pm,
						 viewport,
						 &ox, &oy, &oz);
		_Check_Mouse_Results(ox, oy);
	}
	glutPostRedisplay();				// Update info_window
	glutSetWindow(main_window);
	glutPostRedisplay();				// Update main_window
} // End of _Static_Mouse()


// Moving Measuring-Cross, detects Mouse-Moving while Button pressed
// ---------------------------------------------------------------------------
static void Moved_Mouse(int x, int y);
static void Moved_Mouse(int x, int y) {
static int oldx=0, oldy=0;
GLfloat delta=1.0f/(GLfloat)grids;

	if(x>oldx) {
		oldx=x;
		crossX+=delta;
		if(crossX>1.0f) crossX=1.0f;
	}
   else {
		oldx=x;
	   crossX-=delta;
		if(crossX<-1.0f) crossX=-1.0f;
	}
	if(y>oldy) {
		oldy=y;
		crossY-=delta;
		if(crossY<-1.0f) crossY=-1.0f;
	}
	else {
		oldy=y;
	   crossY+=delta;
		if(crossY>1.0f) crossY=1.0f;
	}
	glutPostRedisplay();
	glutSetWindow(info_window);
	glutPostRedisplay();
	glutSetWindow(main_window);
} // End of Moved_Mouse()


// Evaluating "normal" Keyboard-Actions, DEL is handled as Asci 127
// ---------------------------------------------------------------------------
static void Key(unsigned char key, int x, int y);
static void Key(unsigned char key, int x, int y) {
static int mode=0;
	glutSetWindow(main_window);
	switch(key) {
		case 27:        				// Exit on Escape ...
			exit (0);
			break;
		case '1':						// Input Function 1 ...
			printf("f1(x) = ");
			gets(f1);
			prepare_input(f1);
			mode_func=0;
			update=true;
			break;
		case '2':						// Input Function 2 ...
			printf("f2(x) = ");
			gets(f2);
			prepare_input(f2);
			mode_func=1;
			update=true;
			break;
		case '3':						// Input Function 3 ...
			printf("f3(x) = ");
			gets(f3);
			prepare_input(f3);
			mode_func=2;
			update=true;
			break;
		case ' ':      					// Toggle Fill/Line-Mode ...
			if(PolygonMode==GL_FILL)
				PolygonMode=GL_LINE;
			else
				PolygonMode=GL_FILL;
			glPolygonMode(GL_FRONT_AND_BACK, PolygonMode);
			break;
		case '+':        				// Zoom In ...
			angle-=2.0;
			Reshape(ww,wh);
			break;
		case '-':        				// Zoom Out ...
			angle+=2.0;
			Reshape(ww,wh);
			break;
		case 'a':        				// Toggle Axes On/Off ...
		case 'A':
			mode_axes=!mode_axes;
			break;
		case 'c':        				// Toggle Cross On/off ...
		case 'C':
			mode_cross=!mode_cross;
			break;
		case 'd':        				// Run Demo/Benchmark ...
		case 'D':
			Demo();
			break;
		case 'g':        				// Decrease Tesselation-Grid ...
			step*=2.0;
			break;
    	case 'G':        				// Increase Tesselation-Grid ...
      	step/=2.0;
      	break;
    	case 'm':        				// Toggle draw-mode als follows:
    	case 'M':						// 0=Triangles 1=Lines 2=Points
			if(++mode_draw==3)
	      	mode_draw=0;
			update=true;
			break;
		case 'r':						// Reset to Default-Values
		case 'R':
			Set_Default();
			Reshape(ww,wh);
			break;
		case 's':						// Equation-System-Solver
		case 'S':
			Solve();
			break;
		case 't':						// Toggle Functions drawn
		case 'T':						// 0=F1, 1=F2, 2=F3, 3=F1+F2+F3
			if(++mode_func==4)
				mode_func=0;
			update=true;
			break;
		case 'x':        				// Scale X-Axis ...
			sx/=2.0;
			update=true;
			break;
		case 'X':
			sx*=2.0;
			update=true;
			break;
		case 'y':        				// Scale Y-Axis ...
			sy/=2.0;
			update=true;
			break;
		case 'Y':
			sy*=2.0;
			update=true;
			break;
		case 'z':        				// Scale Z-Axis ...
			sz/=1.05;
			update=true;
			break;
		case 'Z':
			sz*=1.05;
			update=true;
			break;
		case 127:        				// Rotate around Z-axe ... DEL is "normal" Key
			rotZ -= DELTA_ROT;
			break;
		case 'F':
		case 'f':        				// Toggle Window FullScreen ...
			if(mode=!mode)
				glutFullScreen();
			else
				glutReshapeWindow(WINSIZE+185, WINSIZE);
			break;
		default:        				// Something else
		break;
	}
	glutSetWindow(info_window);
	glutPostRedisplay();
	glutSetWindow(main_window);
	glutPostRedisplay();
} // End of Key()


// Handles "special" Keyboard-Actions like Arrow-Keys, ...
// ---------------------------------------------------------------------------
static void Special_Key(int key, int x, int y);
static void Special_Key(int key, int x, int y) {
	switch(key) {
		case GLUT_KEY_F1:
			Show_Help();
			break;
		case GLUT_KEY_UP:			// X-Rotation
			rotX -= DELTA_ROT;
			break;
		case GLUT_KEY_DOWN:
			rotX += DELTA_ROT;
			break;
		case GLUT_KEY_LEFT:			// Y-Rotation
			rotY -= DELTA_ROT;
			break;
		case GLUT_KEY_RIGHT:
			rotY += DELTA_ROT;
			break;
		case GLUT_KEY_PAGE_DOWN:	// Z-Rotation
			rotZ += DELTA_ROT;
			break;
		case GLUT_KEY_PAGE_UP:		// X-Translation
			transX += DELTA_TRANS;
			break;
		case 108:					// Insert
			transX -= DELTA_TRANS;
			break;
		case GLUT_KEY_END:			// Y-Translation
			transY -= DELTA_TRANS;
			break;
		case GLUT_KEY_HOME:
			transY += DELTA_TRANS;
			break;
		default:					// Something else
		break;
	}
	glutSetWindow(main_window);
	glutPostRedisplay();
} // End of SpecialKey


/* ************************************************************************* */
/* Last but not least the Main-Body                                          */
/* ************************************************************************* */
int main(int argc, char **argv) {

	// Doing some Initializations
	strcpy(f1,"COS(4X^2+4Y^2)/(1.5+X+3Y^2)-0.5\0");
	strcpy(f2,"(X+0.3)*Y-0.25\0");
	strcpy(f3,"XXY-0.4\0");

	glutInit(&argc, argv);
	glutInitWindowPosition(150,0);
	glutInitWindowSize(WINSIZE+185, WINSIZE);

	// Get Rendering-Contexts for Z-Buffer, Doublebuffer for RGB-Mode
	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);

	// Create Main-Window
	main_window=glutCreateWindow("Funky3D Version 1.1");
	if(main_window==GL_FALSE) exit(1);
	Init();

	// Define Main-Window-Callbacks
	glutReshapeFunc(Reshape);
	glutKeyboardFunc(Key);
	glutSpecialFunc(Special_Key);
	glutDisplayFunc(Display);
	glutMotionFunc(Moved_Mouse);

	// Create Info-Window
	info_window=glutCreateSubWindow(main_window, 0, 0, 185, 960);
	if(info_window==GL_FALSE) exit(1);
	_Init();

	// Define Info-Window-Callbacks
	glutReshapeFunc(_Reshape);
	glutDisplayFunc(_Display);
	glutSpecialFunc(Special_Key);
	glutMouseFunc(_Static_Mouse);
	glutKeyboardFunc(Key);

	// And finally loop forever ...
	glutMainLoop();
	return (0);
} // End of main()
