/* Funky3D 1.1
 *
 * Copyright (C) 1991, 1992, 2002, 2003 Heinz van Saanen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Under Windows-Platforms, Funky3D must be compiled as console-application       */
/* ============================================================================== */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "ogl_parser.h"

static FTYPE expression(void);
static FTYPE term(void);
static void factor(FTYPE *value);
static FTYPE fak(FTYPE value);
static FTYPE NPV(FTYPE kzf);

static char *cp;					// Global char-pointer for parsed String

// Central parsing-function
// ============================================================================
extern FTYPE parse(char s[]) {
	cp=s;		// Set global "parse" pointer
	elevel=0;
	if(*cp=='\0') return (0.0);
	return (expression());
} // End of parse()


// Calculating expression
// ============================================================================
static FTYPE expression(void);
static FTYPE expression(void) {
FTYPE value;

	value=term();
	if(elevel) return 0.0;
	while((*cp=='+') || (*cp=='-'))
		if(*cp++=='+') {
			value+=term();
			if(elevel) return 0.0;
		}
		else {
			value-=term();
			if(elevel) return 0.0;
		}
	return (value);
} // End of expression()


// Calculating terms
// ============================================================================
static FTYPE term(void);
static FTYPE term(void) {
FTYPE value, help;

	factor(&value);
	if(elevel) return 0.0;
	while(*cp=='*' || *cp=='/' || isalpha(*cp) || *cp=='(')
		if(*cp=='*') {
			cp++;
			factor(&help);
			if(elevel) return (0.0);
			value*=help;
		}
		else if(isalpha(*cp) || *cp=='(') {
			factor(&help);
			if(elevel) return (0.0);
			value*=help;
		}
		else {
			cp++;
			factor(&help);
			if(elevel) return (0.0);
			if(help==0.0) { elevel=1; show_error(elevel); return 0.0; }
			value/=help;
		}
	return (value);
} // End of term()


// Handling unitary Operators
// ============================================================================
#define CHECK(); \
if(*cp=='!') { *value=fak(*value); cp++; } \
if(*cp=='^') { cp++; factor(&help); *value=pow(*value,help); } \
if(*cp=='%') { cp++; factor(&help); if(help>0.0) *value=fmod(*value,help); \
else { elevel=2; }} \
if(isdigit(*cp)) *value*=expression();	// handles x2, sin(x)2, ...


// Calculating Factor-Values
// ============================================================================
static void factor(FTYPE *value);
static void factor(FTYPE *value) {
char doublestr[30];					// Memory for number-extraction
char *dspt=doublestr; 				// Pointer to actual position in doublestring
FTYPE help;

	if(elevel) { *value=0.0; return; }
	if(isdigit(*cp) || *cp=='.') {					// Starting a number?
		while(isdigit(*cp)||*cp=='.')				// Then copy until not number or decimalpoint
			*dspt++=*cp++;
		if(*cp=='E' && !isalpha(*(cp+1))) {			// Exponent following?
			*dspt++=*cp++;							// Then go on copying
			if(*cp=='+'||*cp=='-'||isdigit(*cp)) {	// Check Exponent-Sign
	       	*dspt++=*cp++;							// And go on Copying
				while(isdigit(*cp)) *dspt++=*cp++;	// Until Exponent-Numbers finished
			}
		}
		*dspt='\0';									// Close "number-part" of string
		*value=atof(doublestr);						// Float conversion finished
		CHECK();									// Check for following unitary Operator ! , ^ and %
	}												// From here on check everthing not a Number
	else	if(*cp=='-') {							// Handles leading Minus
				cp++;
				factor(&help);
				*value=-help;
			}
	else	if(*cp=='+') {							// Handles leading Plus
				cp++;
				*value=expression();
				cp++;
			}
	else	if(*cp=='(') {							// Handles open Bracket
				cp++;
				*value=expression();
				cp++;
				CHECK();							// Check for following unitary Operator ! , ^ and %
			}
	else	if(!strncmp("X",cp,1)) {				// Handles global X
				cp++;
				*value=x;
				CHECK();
			}
	else	if(!strncmp("Y",cp,1)) {				// Handles global Y
				cp++;
				*value=y;
				CHECK();
			}
	else	if(!strncmp("SIN(",cp,4)) {				// Handles sin(...)
				cp+=4;
				*value=sin(expression());
				cp++;
				CHECK();
			}
	else	if(!strncmp("COS(",cp,4)) {				// Handles cos(...)
				cp+=4;
				*value=cos(expression());
				cp++;
			}
	else	if(!strncmp("TAN(",cp,4)) {				// Handles tan(...)
				cp+=4;
				*value=tan(expression());
				cp++;
			}
	else	if(!strncmp("SQRT(",cp,5)) {			// Handles sqrt(...)
				cp+=5;
				*value=expression(); if(*value>0.0) *value=sqrt(*value); else { elevel=5; *value=0.0f; }
				//*value=sqrt(expression());
				cp++;
				CHECK();
			}
	else	if(!strncmp("PI",cp,2)) {				// Handles PI=3.1415...
				cp+=2;								// Offset just 2 because of constant Value
				*value=M_PI;						// Constant! don't increment Pointer
				CHECK();
			}
	else	if(!strncmp("ABS(",cp,4)) {				// Handles abs(...)
				cp+=4;
				*value=fabs(expression());
				cp++;
				CHECK();
			}
	else	if(!strncmp("LOG(",cp,4)) {				// Handles natural log(...)
				cp+=4;
				*value=expression();
				if(*value>0.0) *value=log(*value); else { elevel=6; *value=0.0f; }
				cp++;
			}
	else	if(!strncmp("LOG10(",cp,6)) {			// Handles log10(...)
				cp+=6;
				*value=expression();
				if(*value>0.0) *value=log10(*value); else { elevel=6; *value=0.0f; }
				cp++;
			}
	else	if(!strncmp("EULER",cp,5)) {			// Handles Euler-Constant E=2.7182...
				cp+=5;								// Offset just 5 because of constant Value
				*value=M_E;							// Constant! don't increment Pointer
				CHECK();
			}
	else	if(!strncmp("NPV(",cp,4)) {				// Handles Net Present Value (NPV)
				cp+=4;								// For Solving Equations of
				if(npvloaded==true)					// Internal Rates of Return (IRR)
				   *value=NPV(expression());		// Used in 2D-Version of Funky3D only
				else
				   *value=0.0;
				cp++;
			}
	else	{										// Else something "strange" occured
		elevel=3;
		show_error(elevel);
		return;
	}
} // End of factor()


/* ************************************************************************* */
/*  Some Utility-Functions                                                   */
/* ************************************************************************* */

// Kill all Blanks, returns Upper Cases
// ============================================================================
extern int prepare_input(char *cpt) {
  register char *help=cpt;
  if(*cpt=='\0') return false;
  while(*cpt==' ') cpt++;
  while(*help++=toupper(*cpt++)) while(*cpt==' ') cpt++;
  return true;
} // End of prepare_input()


// Checks Brackets
// ============================================================================
extern int check_brackets(char *cpt) {
  register char *help=cpt;
  register int open=0, close=0;
  while(*help) {
	  if(*help=='(') open++;
	  else if(*help==')') close++;
	  help++;
  }
  return open==close;
} // End of check_brackets()


// Calculates Faculty. Returns 1.0 for 0!
// ============================================================================
static FTYPE fak(FTYPE value);
static FTYPE fak(FTYPE value) {
  long int i,j;
  FTYPE k=1.0;
  j=(long int)rint(value);
  if(j>=0) for(i=2; i<=j; i++) k*=(FTYPE)i;
  else { elevel=4; return 0.0; } 	// Error-handling
  return (k);
} // End of fak()


// For some Cashflow-Analysis; used in 2D-Version of Funky3D only
// ===========================================================
#define INDEX 256			// Maximum Index for cashflows to read
FTYPE inpt[INDEX][2];		// Field for discounted Cash-Flows
int cfn;					// Number of Cash-Flows for NPV()
int line;					// "Pointer" to actual Line of File-Input
int maxline=0;				// Lines for Input-File


// Calculates with Data in inpt [0..cfn-1][1]
// kzf = Interest Rate for discounted Cashflows
static FTYPE NPV(FTYPE kzf);
static FTYPE NPV(FTYPE kzf) {
FTYPE su=inpt[0][1], help1;
int j;
	if((kzf<=-100.0)) return 0.0;
	kzf/=100.0;
	for(j=1; j<cfn; j++) {
		help1=pow(1.0+kzf,j);
		if(help1==0) return 0.0;
		help1=1.0/help1;
		su+=inpt[j][1]*help1;
	}
	return su;
} // End of NPV()


// Loading NPV-File; used in 2D-Version of Funky3D only
// ===========================================================
extern void opencsv(char *s) {
FILE *fp;
int STRLEN=80;
char c1[STRLEN];

	npvloaded=false;
	for(line=0; line<INDEX; line++) inpt[line][1]='\0';
	line=0;

	if((fp=fopen(s,"r"))==NULL) {
		printf("Unable to open file %s\n", s);
		return;
	}
	else {
		while(fgets(&c1[line], STRLEN, fp) != NULL) {
			inpt[line][1]=atof(&c1[line]);
			line++;
		}
		npvloaded=true;
		printf("File %s loaded\n", s);
	}
	cfn=maxline=line;
	line=0;
	fclose(fp);
} // End of opencsv()


// Error-Handling
// ===========================================================
void show_error(int number) {
	switch (number) {
		case 1: printf("ERROR 1: Division by Zero\n"); break;
		case 2: printf("ERROR 2: Modulus is Zero\n"); break;
		case 3: printf("ERROR 3: Unknown Expression\n"); break;
		case 4: printf("ERROR 4: Faculty is negative\n"); break;
		case 5: printf("ERROR 5: Root is negative\n"); break;
		case 6: printf("ERROR 6: Logarithm is negative\n"); break;
		default: break;
	}
}


